@@ -1,22 +1,9 @@
-/***************************************************************************
- *   This file is part of KDevelop                                         *
- *   Copyright 2008 Niko Sams <niko.sams@gmail.com>                        *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
- *   published by the Free Software Foundation; either version 2 of the    *
- *   License, or (at your option) any later version.                       *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
- *   License along with this program; if not, write to the                 *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
- ***************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2008 Niko Sams <niko.sams@gmail.com>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
+
 #include "helper.h"
 
 #include <KParts/MainWindow>
@@ -30,6 +17,7 @@
 #include <language/duchain/parsingenvironment.h>
 #include <language/duchain/types/unsuretype.h>
 #include <language/duchain/types/integraltype.h>
+#include <language/duchain/types/arraytype.h>
 #include <interfaces/icore.h>
 #include <interfaces/iprojectcontroller.h>
 #include <interfaces/iuicontroller.h>
@@ -44,7 +32,6 @@
 #include "declarations/classdeclaration.h"
 #include "declarations/classmethoddeclaration.h"
 #include "declarations/functiondeclaration.h"
-#include "types/indexedcontainer.h"
 #include "types/integraltypeextended.h"
 #include "expressionparser.h"
 #include "expressionvisitor.h"
@@ -86,6 +73,28 @@ bool isMatch(Declaration* declaration, DeclarationType declarationType)
     return false;
 }
 
+bool isGenericClassTypehint(NamespacedIdentifierAst* node, EditorIntegrator *editor)
+{
+    const KDevPG::ListNode< IdentifierAst* >* it = node->namespaceNameSequence->front();
+    QString typehint = editor->parseSession()->symbol(it->element);
+
+    if (typehint.compare(QLatin1String("bool"), Qt::CaseInsensitive) == 0) {
+        return false;
+    } else if (typehint.compare(QLatin1String("float"), Qt::CaseInsensitive) == 0) {
+        return false;
+    } else if (typehint.compare(QLatin1String("int"), Qt::CaseInsensitive) == 0) {
+        return false;
+    } else if (typehint.compare(QLatin1String("string"), Qt::CaseInsensitive) == 0) {
+        return false;
+    } else if (typehint.compare(QLatin1String("iterable"), Qt::CaseInsensitive) == 0) {
+        return false;
+    } else if (typehint.compare(QLatin1String("object"), Qt::CaseInsensitive) == 0) {
+        return false;
+    } else {
+        return true;
+    }
+}
+
 bool isClassTypehint(GenericTypeHintAst* genericType, EditorIntegrator *editor)
 {
     Q_ASSERT(genericType);
@@ -95,25 +104,20 @@ bool isClassTypehint(GenericTypeHintAst* genericType, EditorIntegrator *editor)
     } else if (genericType->arrayType != -1) {
         return false;
     } else if (genericType->genericType) {
-        NamespacedIdentifierAst* node = genericType->genericType;
-        const KDevPG::ListNode< IdentifierAst* >* it = node->namespaceNameSequence->front();
-        QString typehint = editor->parseSession()->symbol(it->element);
+        return isGenericClassTypehint(genericType->genericType, editor);
+    } else {
+        return false;
+    }
+}
 
-        if (typehint.compare(QLatin1String("bool"), Qt::CaseInsensitive) == 0) {
-            return false;
-        } else if (typehint.compare(QLatin1String("float"), Qt::CaseInsensitive) == 0) {
-            return false;
-        } else if (typehint.compare(QLatin1String("int"), Qt::CaseInsensitive) == 0) {
-            return false;
-        } else if (typehint.compare(QLatin1String("string"), Qt::CaseInsensitive) == 0) {
-            return false;
-        } else if (typehint.compare(QLatin1String("iterable"), Qt::CaseInsensitive) == 0) {
-            return false;
-        } else if (typehint.compare(QLatin1String("object"), Qt::CaseInsensitive) == 0) {
-            return false;
-        } else {
-            return true;
-        }
+bool isClassTypehint(PropertyTypeHintAst* propertyType, EditorIntegrator *editor)
+{
+    Q_ASSERT(propertyType);
+
+    if (propertyType->arrayType != -1) {
+        return false;
+    } else if (propertyType->genericType) {
+        return isGenericClassTypehint(propertyType->genericType, editor);
     } else {
         return false;
     }
@@ -215,72 +219,45 @@ DeclarationPointer findDeclarationImportHelper(DUContext* currentContext, const
 
 DeclarationPointer findDeclarationInPST(DUContext* currentContext, QualifiedIdentifier id, DeclarationType declarationType)
 {
+    DeclarationPointer ret;
+
     ifDebug(qCDebug(DUCHAIN) << "PST: " << id.toString() << declarationType;)
-    uint nr;
-    const IndexedDeclaration* declarations = nullptr;
     DUChainWriteLocker wlock;
-    PersistentSymbolTable::self().declarations(id, nr, declarations);
-    ifDebug(qCDebug(DUCHAIN) << "found declarations:" << nr;)
+
     /// Indexed string for 'Php', identifies environment files from this language plugin
     static const IndexedString phpLangString("Php");
-
-    for (uint i = 0; i < nr; ++i) {
-        ParsingEnvironmentFilePointer env = DUChain::self()->environmentFileForDocument(declarations[i].indexedTopContext());
+    auto visitor = [&](const IndexedDeclaration &indexedDeclaration) {
+        ParsingEnvironmentFilePointer env = DUChain::self()->environmentFileForDocument(indexedDeclaration.indexedTopContext());
         if(!env) {
             ifDebug(qCDebug(DUCHAIN) << "skipping declaration, missing meta-data";)
-            continue;
+            return PersistentSymbolTable::VisitorState::Continue;
         }
         if(env->language() != phpLangString) {
             ifDebug(qCDebug(DUCHAIN) << "skipping declaration, invalid language" << env->language().str();)
-            continue;
+            return PersistentSymbolTable::VisitorState::Continue;
         }
 
-        if (!declarations[i].declaration()) {
+        const auto declaration = indexedDeclaration.declaration();
+        if (!declaration) {
             ifDebug(qCDebug(DUCHAIN) << "skipping declaration, doesn't have declaration";)
-            continue;
-        } else if (!isMatch(declarations[i].declaration(), declarationType)) {
+            return PersistentSymbolTable::VisitorState::Continue;
+        } else if (!isMatch(declaration, declarationType)) {
             ifDebug(qCDebug(DUCHAIN) << "skipping declaration, doesn't match with declarationType";)
-            continue;
+            return PersistentSymbolTable::VisitorState::Continue;
         }
-        TopDUContext* top = declarations[i].declaration()->context()->topContext();
-
-        /*
-            * NOTE:
-            * To enable PHPUnit test classes, this check has been disabled.
-            * Formerly it only loaded declarations from open projects, but PHPUnit declarations
-            * belong to no project.
-            *
-            * If this behavior is unwanted, reinstate the check.
-            * Miha Cancula <miha@noughmad.eu>
-            */
-        /*
-        if (ICore::self() && !ICore::self()->projectController()->projects().isEmpty()) {
-            bool loadedProjectContainsUrl = false;
-            foreach(IProject *project, ICore::self()->projectController()->projects()) {
-                if (project->fileSet().contains(top->url())) {
-                    loadedProjectContainsUrl = true;
-                    break;
-                }
-            }
-            if (!loadedProjectContainsUrl) {
-                ifDebug(qCDebug(DUCHAIN) << "skipping declaration, not in loaded project";)
-                continue;
-            }
-        }
-        */
 
+        TopDUContext* top = declaration->context()->topContext();
         currentContext->topContext()->addImportedParentContext(top);
         currentContext->topContext()->parsingEnvironmentFile()
         ->addModificationRevisions(top->parsingEnvironmentFile()->allModificationRevisions());
         currentContext->topContext()->updateImportsCache();
         ifDebug(qCDebug(DUCHAIN) << "using" << declarations[i].declaration()->toString() << top->url();)
-        wlock.unlock();
-        return DeclarationPointer(declarations[i].declaration());
-    }
+        ret = declaration;
+        return PersistentSymbolTable::VisitorState::Break;
+    };
+    PersistentSymbolTable::self().visitDeclarations(id, visitor);
 
-    wlock.unlock();
-    ifDebug(qCDebug(DUCHAIN) << "returning 0";)
-    return DeclarationPointer();
+    return ret;
 }
 
 QByteArray formatComment(AstNode* node, EditorIntegrator* editor)
@@ -481,7 +458,7 @@ AbstractType::Ptr determineTypehint(const T* genericType, EditorIntegrator *edit
             } else if (typehint.compare(QLatin1String("object"), Qt::CaseInsensitive) == 0) {
                 type = AbstractType::Ptr(new IntegralTypeExtended(IntegralTypeExtended::TypeObject));
             } else if (typehint.compare(QLatin1String("iterable"), Qt::CaseInsensitive) == 0) {
-                DeclarationPointer traversableDecl = findDeclarationImportHelper(currentContext, QualifiedIdentifier("traversable"), ClassDeclarationType);
+                DeclarationPointer traversableDecl = findDeclarationImportHelper(currentContext, QualifiedIdentifier(u"traversable"), ClassDeclarationType);
 
                 if (traversableDecl) {
                     UnsureType::Ptr unsure(new UnsureType());
@@ -504,15 +481,15 @@ AbstractType::Ptr determineTypehint(const T* genericType, EditorIntegrator *edit
 
     if (type && genericType->isNullable != -1) {
         AbstractType::Ptr nullType = AbstractType::Ptr(new IntegralType(IntegralType::TypeNull));
-        if (type.cast<UnsureType>()) {
-            UnsureType::Ptr unsure = type.cast<UnsureType>();
+        auto unsure = type.dynamicCast<UnsureType>();
+        if (unsure) {
             unsure->addType(nullType->indexed());
         } else {
-            UnsureType::Ptr unsure(new UnsureType());
+            unsure = UnsureType::Ptr(new UnsureType());
             unsure->addType(type->indexed());
             unsure->addType(nullType->indexed());
 
-            type = AbstractType::Ptr(unsure);
+            type = unsure;
         }
     }
 
@@ -537,17 +514,17 @@ AbstractType::Ptr parameterType(const ParameterAst* node, AbstractType::Ptr phpD
          *    so we can ignore this case.
          * If it's null (which cannot be a typehint), add it as UnsureType
          */
-        if (type && defaultValueType.cast<IntegralType>() && defaultValueType.cast<IntegralType>()->dataType() == IntegralType::TypeNull) {
-            if (type.cast<UnsureType>()) {
-                UnsureType::Ptr unsure = type.cast<UnsureType>();
+        if (type && defaultValueType.dynamicCast<IntegralType>() && defaultValueType.staticCast<IntegralType>()->dataType() == IntegralType::TypeNull) {
+            auto unsure = type.dynamicCast<UnsureType>();
+            if (unsure) {
                 AbstractType::Ptr nullType = AbstractType::Ptr(new IntegralType(IntegralType::TypeNull));
                 unsure->addType(defaultValueType->indexed());
             } else {
-                UnsureType::Ptr unsure(new UnsureType());
+                unsure = UnsureType::Ptr(new UnsureType());
                 unsure->addType(type->indexed());
                 unsure->addType(defaultValueType->indexed());
 
-                type = AbstractType::Ptr(unsure);
+                type = unsure;
             }
         } else {
             //Otherwise, let the default value dictate the parameter type
@@ -566,13 +543,12 @@ AbstractType::Ptr parameterType(const ParameterAst* node, AbstractType::Ptr phpD
       ReferenceType::Ptr p( new ReferenceType() );
       p->setBaseType( type );
 
-      type = p.cast<AbstractType>();
+      type = p;
     }
 
     if (node->isVariadic != -1) {
-        auto *container = new IndexedContainer();
-        container->addEntry(type);
-        container->setPrettyName(IndexedString("array"));
+        auto *container = new KDevelop::ArrayType();
+        container->setElementType(type);
         type = AbstractType::Ptr(container);
     }
 
@@ -580,6 +556,25 @@ AbstractType::Ptr parameterType(const ParameterAst* node, AbstractType::Ptr phpD
     return type;
 }
 
+AbstractType::Ptr propertyType(const ClassStatementAst* node, AbstractType::Ptr phpDocTypehint, EditorIntegrator* editor, DUContext* currentContext)
+{
+    AbstractType::Ptr type;
+    if (node->propertyType) {
+        type = determineTypehint(node->propertyType, editor, currentContext);
+    }
+
+    if (!type) {
+        if (phpDocTypehint) {
+            type = phpDocTypehint;
+        } else {
+            type = AbstractType::Ptr(new IntegralType(IntegralType::TypeMixed));
+        }
+    }
+
+    Q_ASSERT(type);
+    return type;
+}
+
 AbstractType::Ptr returnType(const ReturnTypeAst* node, AbstractType::Ptr phpDocTypehint, EditorIntegrator* editor, DUContext* currentContext) {
     AbstractType::Ptr type;
     if (node) {
