@@ -1,21 +1,8 @@
-/* This file is part of KDevelop
+/*
+    SPDX-FileCopyrightText: 2010 Niko Sams <niko.sams@gmail.com>
+    SPDX-FileCopyrightText: 2011 Milian Wolff <mail@milianw.de>
 
-   Copyright 2010 Niko Sams <niko.sams@gmail.com>
-   Copyright 2011 Milian Wolff <mail@milianw.de>
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-only
 */
 
 #include "duchain_multiplefiles.h"
@@ -200,7 +187,7 @@ void TestDUChainMultipleFiles::testForeachImportedIdentifier()
 
     for(int i = 0; i < 2; ++i) {
         if (i > 0) {
-            features = static_cast<TopDUContext::Features>(features | TopDUContext::ForceUpdate);
+            features |= TopDUContext::ForceUpdate;
         }
         f2.parse(features);
         QVERIFY(f2.waitForParsed());
@@ -240,7 +227,7 @@ void TestDUChainMultipleFiles::testUpdateForeach()
     {
         DUChainWriteLocker lock;
         QVERIFY(f.topContext()->problems().isEmpty());
-        QCOMPARE(f.topContext()->findDeclarations(Identifier("k")).count(), 1);
+        QCOMPARE(f.topContext()->findDeclarations(Identifier(u"k")).count(), 1);
         Declaration* kDec = f.topContext()->findDeclarations(Identifier(QStringLiteral("k"))).first();
         QCOMPARE(kDec->rangeInCurrentRevision().start().line(), 1);
         QCOMPARE(kDec->rangeInCurrentRevision().start().column(), 0);
@@ -251,14 +238,14 @@ void TestDUChainMultipleFiles::testUpdateForeach()
 
     // delete $k = null; line
     f.setFileContents(QStringLiteral("<?\nforeach(array() as $i => $k) {}\n"));
-    f.parse(static_cast<TopDUContext::Features>(features | TopDUContext::ForceUpdate));
+    f.parse(features | TopDUContext::ForceUpdate);
     QVERIFY(f.waitForParsed());
     QVERIFY(f.topContext());
 
     {
         DUChainWriteLocker lock;
         QVERIFY(f.topContext()->problems().isEmpty());
-        QCOMPARE(f.topContext()->findDeclarations(Identifier("k")).count(), 1);
+        QCOMPARE(f.topContext()->findDeclarations(Identifier(u"k")).count(), 1);
         Declaration* kDec = f.topContext()->findDeclarations(Identifier(QStringLiteral("k"))).first();
         QCOMPARE(kDec->rangeInCurrentRevision().start().line(), 1);
         QCOMPARE(kDec->rangeInCurrentRevision().start().column(), 25);
@@ -272,12 +259,12 @@ void TestDUChainMultipleFiles::testTodoExtractorReparse()
 
     QVERIFY(KDevelop::ICore::self()->languageController()->completionSettings()->todoMarkerWords().contains("TODO"));
 
-    auto features = TopDUContext::AllDeclarationsContextsAndUses;
+    TopDUContext::Features features{TopDUContext::AllDeclarationsContextsAndUses};
 
     for (int i = 0; i < 2; ++i) {
         if (i == 1) {
             file.setFileContents(QStringLiteral("<?php\n$foo = new bar();\n// TODO\n$foo->asdf();"));
-            features = static_cast<TopDUContext::Features>(features | TopDUContext::ForceUpdate);
+            features |= TopDUContext::ForceUpdate;
         }
 
         file.parse(features);
@@ -295,12 +282,12 @@ void TestDUChainMultipleFiles::testTodoExtractorReparse()
 void TestDUChainMultipleFiles::testIteratorForeachReparse() {
     TestFile file(QStringLiteral("<?php\n/*\n\n/*\n*/\nforeach (new A() as $a) {}\nclass A implements Iterator {\npublic function current() { return 0; }\n}"), QStringLiteral("php"));
 
-    auto features = TopDUContext::AllDeclarationsAndContexts;
+    TopDUContext::Features features{TopDUContext::AllDeclarationsAndContexts};
 
     for (int i = 0; i < 2; ++i) {
         if (i == 1) {
             file.setFileContents(QStringLiteral("<?php\n/*\n*/\n\n/*\n*/\nforeach (new A() as $a) {}\nclass A implements Iterator {\npublic function current() { return 0; }\n}"));
-            features = static_cast<TopDUContext::Features>(features | TopDUContext::ForceUpdate);
+            features |= TopDUContext::ForceUpdate;
         }
 
         file.parse(features);
@@ -310,7 +297,7 @@ void TestDUChainMultipleFiles::testIteratorForeachReparse() {
         auto top = file.topContext();
         QVERIFY(top);
         QVERIFY(top->localDeclarations().size() == 2);
-        QCOMPARE(top->localDeclarations().at(0)->qualifiedIdentifier(), QualifiedIdentifier("a"));
+        QCOMPARE(top->localDeclarations().at(0)->qualifiedIdentifier(), QualifiedIdentifier(u"a"));
 
         IntegralType::Ptr type = top->localDeclarations().at(0)->type<IntegralType>();
         QVERIFY(type);
@@ -318,3 +305,40 @@ void TestDUChainMultipleFiles::testIteratorForeachReparse() {
         QVERIFY(type->dataType() == IntegralType::TypeMixed);
     }
 }
+
+void TestDUChainMultipleFiles::testNamespacedIdentifierInPST() {
+    constexpr TopDUContext::Features features{TopDUContext::AllDeclarationsAndContexts};
+
+    TestProject* project = new TestProject;
+    m_projectController->closeAllProjects();
+    m_projectController->addProject(project);
+
+    TestFile f1(QStringLiteral("<?\n"
+                "namespace Test;\n"
+                "class A {}\n"), QStringLiteral("php"), project);
+    f1.parse(features);
+    QVERIFY(f1.waitForParsed());
+
+    TestFile f2(QStringLiteral("<?\n"
+                "namespace Test2;\n"
+                "class B {\n"
+                "public $class_a;\n"
+                "public function __construct() { $this->class_a = new \\Test\\A(); }}"), QStringLiteral("php"), project);
+    f2.parse(features);
+    QVERIFY(f2.waitForParsed());
+
+    TestFile f3(QStringLiteral("<?\n"
+                "namespace Test2;\n"
+                "class C {\n"
+                "public $class_a;\n"
+                "public function __construct() { $this->class_a = new Test\\A(); }}"), QStringLiteral("php"), project);
+    f3.parse(features);
+    QVERIFY(f3.waitForParsed());
+
+    DUChainWriteLocker lock(DUChain::lock());
+    QVERIFY(f1.topContext());
+    QVERIFY(f2.topContext());
+    QVERIFY(f2.topContext()->imports(f1.topContext(), CursorInRevision(0, 0)));
+    QVERIFY(f3.topContext());
+    QVERIFY(!f3.topContext()->imports(f1.topContext(), CursorInRevision(0, 0)));
+}
